/*****************************************************************************/
/* EyeClock - A clock application for X. The eyes follow the mouse.          */
/*                                                                           */
/* EyeClock Copyright (c) 1998 Sakai Hiroaki.                                */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* BSD Daemon Copyright 1988 by Marshall Kirk McKusick.                      */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* EyeClock is a simple clock application for X11. You can put an optional   */
/* picture on the face of the clock. And you can move the eyes of the        */
/* picture following the mouse.                                              */
/*                                                                           */
/* EyeClock is a free software. You may copy or distribute the original      */
/* EyeClock freely. But, you may not modify or distribute the original       */
/* EyeClock without permission of the author.                                */
/*                                                                           */
/* Statement on the Use of the BSD Daemon Figure:                            */
/*                                                                           */
/* The BSD Daemon is to be used in the context of BSD software.              */
/* So, if you are using BSD software (BSD/OS, FreeBSD, NetBSD, OpenBSD,      */
/* or BSD utilities) in your Intranet environment, then use of the           */
/* daemon is appropriate. If you are a Microsoft shop, then it is not        */
/* appropriate.                                                              */
/*                                                                           */
/* Individuals may use the daemon for their personal use within the          */
/* bounds of good taste (an example of bad taste was a picture of the        */
/* BSD daemon blowtorching a Solaris logo). When reasonably possible,        */
/* I would like the text ``BSD Daemon Copyright 1988 by Marshall Kirk        */
/* McKusick. All Rights Reserved.'' to be included.  This text need          */
/* not be etched into the figure or garishly displayed when using the        */
/* daemon as say an Icon in a Web window. A good example of how              */
/* to handle the due credit in a web page is to create a link from           */
/* the daemon picture to the following text:                                 */
/*                                                                           */
/*         BSD Daemon Copyright 1988 by Marshall Kirk McKusick.              */
/*         All Rights Reserved.                                              */
/*                                                                           */
/*         Permission to use the daemon may be obtained from:                */
/*                 Marshall Kirk McKusick                                    */
/*                 1614 Oxford St                                            */
/*                 Berkeley, CA 94709-1608                                   */
/*                 USA                                                       */
/*         or via email at mckusick@mckusick.com                             */
/*                                                                           */
/* If you are looking for daemon images, a pictorial history of the          */
/* daemon can be found at http://www.mckusick.com/beastie/index.html.        */
/*                                                                           */
/* If you want to mass produce the daemon on Tshirts, CDROM's, or other      */
/* products you need to request permission in advance. In general, I require */
/* that the daemon be used in an appropriate way. This means that it         */
/* has to be something related to BSD and not expropriated as a company      */
/* logo (though I do allow companies with BSD-based products such as         */
/* Walnut Creek CDROM or BSDI to use it).  I regret having to be so          */
/* legalistic about the daemon, but I almost lost the daemon to a            */
/* certain large company because I failed to show due diligence in           */
/* protecting it. So, I've taken due diligence seriously since then.         */
/*                                                                           */
/*         Marshall Kirk McKusick                                            */
/*****************************************************************************/

/*****************************************************************************/
/* إåΥ󥯥롼                                                      */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<time.h>
#include<math.h>
#include<X11/Xlib.h>
#include<X11/Intrinsic.h>
#include<X11/StringDefs.h>
#include<X11/keysym.h>
#include<X11/xpm.h>
#include<X11/Xaw/Label.h>

#include"configure.h"

/*****************************************************************************/
/* ǥեȤΥǡ                                                  */
/*****************************************************************************/

#define DEFAULT_PIXMAP sample1
#define DEFAULT_DATA   sample1_eye
#include"sample1.xpm"
static char * DEFAULT_DATA =
"# Eye Clock Data (the BSD daemon)\n"
"# Pixmap			sample1.xpm\n"
"Name				DaemonClock\n"
"Width				128\n"
"Height				128\n"
"ClockFlag			ON\n"
"EyeFlag			ON\n"
"EveryFlag			OFF\n"
"DefaultEyeForegroundColor	black\n"
"DefaultEyeBackgroundColor	white\n"
"LongForegroundColor		green\n"
"LongBackgroundColor		black\n"
"ShortForegroundColor		green\n"
"ShortBackgroundColor		black\n"
"LongLength			60\n"
"LongForegroundThickness	3\n"
"LongBackgroundThickness	5\n"
"ShortLength			40\n"
"ShortForegroundThickness	6\n"
"ShortBackgroundThickness	8\n"
"Center				64 64\n"
"\n"
"NewEye\n"
"EyePoint		39 32\n"
"EyeCenter		2 -1\n"
"EyeMove		100 200\n"
"EyeRadius		6 10\n"
"EyeFigureNumber	19\n"
"EyeFigure		3 0  3 4  2 5  2 7  -1 7  -1 6  -2 5  -2 -1\n"
"			-1 -2  -1 -4  0 -5  0 -6  4 -9  6 -9  6 -8  5 -7\n"
"			5 -5  4 -4  4 -1\n"
"\n"
"NewEye\n"
"EyePoint		50 35\n"
"EyeCenter		0 0\n"
"EyeMove		100 200\n"
"EyeRadius		8 14\n"
"EyeFigureNumber	30\n"
"EyeFigure		-4 0  -5 2  -5 6  -4 7  -4 8  -2 10  0 10  2 9\n"
"			3 9  4 8  4 7  5 6  5 5  6 4  6 2  7 0\n"
"			7 -5  6 -6  6 -8  5 -9  5 -10  4 -11  2 -11  -1 -9\n"
"			-1 -8  -2 -7  -2 -6  -3 -5  -3 -3  -4 -2\n";

/*****************************************************************************/
/* ޤޤ                                                            */
/*****************************************************************************/

#define OFF 0
#define ON  1
#define CLEAR 2
#define DEGTORAD (M_PI/180.0)
#define ANIMATION_NUMBER 3
#define NOT_READ         0
#define READ_FROM_STRING 1
#define READ_FROM_FILE   2

/*****************************************************************************/
/* ǥեȤͤ                                                      */
/*****************************************************************************/

#define DEFAULT_CLOCK_NAME                 "EyeClock"
#define DEFAULT_WIDTH                      100
#define DEFAULT_HEIGHT                     100
#define DEFAULT_CLOCK_FLAG                 ON
#define DEFAULT_EYE_FLAG                   OFF
#define DEFAULT_EVERY_FLAG                 OFF
#define DEFAULT_VERBOSE_FLAG               OFF
#define DEFAULT_EYE_FOREGROUND_COLOR       "black"
#define DEFAULT_EYE_BACKGROUND_COLOR       "white"
#define DEFAULT_LONG_FOREGROUND_COLOR      "blue"
#define DEFAULT_LONG_BACKGROUND_COLOR      "black"
#define DEFAULT_SHORT_FOREGROUND_COLOR     "red"
#define DEFAULT_SHORT_BACKGROUND_COLOR     "black"
#define DEFAULT_LONG_LENGTH                -1
#define DEFAULT_LONG_FOREGROUND_THICKNESS  -1
#define DEFAULT_LONG_BACKGROUND_THICKNESS  -1
#define DEFAULT_SHORT_LENGTH               -1
#define DEFAULT_SHORT_FOREGROUND_THICKNESS -1
#define DEFAULT_SHORT_BACKGROUND_THICKNESS -1
#define DEFAULT_CLOCK_CENTER_X             -1
#define DEFAULT_CLOCK_CENTER_Y             -1

/*****************************************************************************/
/* EyeClockWindow ¤Τ                                             */
/*****************************************************************************/

typedef struct eye_clock_window_struct {
  XtAppContext app_context;
  Display * display;
  Widget toplevel, canvas;
  Window toplevel_window, root_window, clock_window;
  Pixmap back_pixmap, clock_pixmap, figure_pixmap, eye_pixmap, work_pixmap;
  Atom atom_1, atom_2;
  int icon_width, icon_height;
  Pixmap icon_pixmap;
  GC gc_long , gc_long_back , gc_long_clear;
  GC gc_short, gc_short_back, gc_short_clear;
  GC gc_eye_black, gc_eye_white;
  GC gc_copy_copy, gc_copy_and, gc_copy_andinv, gc_copy_or;
} EyeClockWindow;

/*****************************************************************************/
/* Clock ¤Τ                                                      */
/*****************************************************************************/

typedef struct clock_struct {
  int width, height, depth;
  int  long_length,  long_foreground_thickness,  long_background_thickness;
  int short_length, short_foreground_thickness, short_background_thickness;
  XPoint clock_center;
  int clock_flag, eye_flag, every_flag, verbose_flag;
  int animation_flag, animation_type;
  char * data_filename;
  char pixmap_filename[128];
  char clock_name[32];
  char default_eye_foreground_color[32];
  char default_eye_background_color[32];
  char  long_foreground_color[32];
  char  long_background_color[32];
  char short_foreground_color[32];
  char short_background_color[32];
  int read_pixmap_from;
  int read_data_from;
} Clock;

/*****************************************************************************/
/* Eye ¤Τ                                                        */
/*****************************************************************************/

typedef struct eye_struct {
  XPoint point;                  /* ΰ */
  XPoint center;                 /* ̤ܶ濴κɸ */
  XPoint move;                   /* ̤ܶΰưγ(ɴʬΨ) */
  XPoint radius;                 /* ̤ܶ礭 */
  GC gc_eye_foreground;          /* ̤ܶ GC */
  GC gc_eye_background;          /* طʤ GC */
  char eye_foreground_color[32]; /* ̤ܶο */
  char eye_background_color[32]; /* طʤο */
  int number;                    /* ǡ */
  XPoint * figure;               /* سԤΥǡ */
  struct eye_struct * next;      /* ΥǡؤΥݥ */
} Eye;

/*****************************************************************************/
/* Help ɽʸ                                                     */
/*****************************************************************************/

static char * help_string =
"\n"
"        " VERSION " Programed by Sakai Hiroaki\n"
"\n"
"        EyeClock - A clock application for X. The eyes follow the mouse.\n"
"\n"
"SYNOPSIS\n"
"\n"
"        eyeclock [ datafile ] [ -pixmap filename.xpm ] [ -options ... ]\n"
"\n"
"DESCRIPTION\n"
"\n"
"        EyeClock is a simple clock application for X11. You can put an optional\n"
"        picture on the face of the clock. And you can move the eyes of the\n"
"        picture following the mouse.\n"
"\n"
"OPTIONS\n"
"\n"
"        You can use all of the standard X-Toolkit options.\n"
"\n"
"-display [displayname]\n"
"        This option specifies the name of the X server to use.\n"
"\n"
"-geometry [geometry]\n"
"        This option specifies the placement of the window on the screen.\n"
"\n"
"-name [name]\n"
"        This option specifies the name of the application.\n"
"\n"
"-h, -help\n"
"        Output this message.\n"
"\n"
"-clock\n"
"        Draw the hour and the minute hand.\n"
"\n"
"-noclock\n"
"        Not draw the hour and the minute hand.\n"
"\n"
"-eye\n"
"        Move the eyes of the picture.\n"
"\n"
"-noeye\n"
"        Not move the eyes of the picture.\n"
"\n"
"-every\n"
"        Read data every time the mouse moves.\n"
"\n"
"-noevery\n"
"        Read data only the first time.\n"
"\n"
"-v, -verbose\n"
"        Output detailed parameters.\n"
"\n"
"-noverbose\n"
"        Not output detailed parameters.\n"
"\n"
"-pixmap [filename]\n"
"        This option specifies the filename of the pixmap.\n"
"        This option must be written before other options.\n"
"\n"
"-width [n]\n"
"        This option specifies the width of the clock.\n"
"\n"
"-height [n]\n"
"        This option specifies the height of the clock.\n"
"\n"
"-eye-fg-color [color], -eye-foreground-color [color]\n"
"        This option specifies the color of the iris of the eyes.\n"
"\n"
"-eye-bg-color [color], -eye-background-color [color]\n"
"        This option specifies the color of the white of the eyes.\n"
"\n"
"-long-fg-color [color], -long-foreground-color [color]\n"
"        This option specifies the color of the minute hand.\n"
"\n"
"-long-bg-color [color], -long-background-color [color]\n"
"        This option specifies the background color of the minute hand.\n"
"\n"
"-short-fg-color [color], -short-foreground-color [color]\n"
"        This option specifies the color of the hour hand.\n"
"\n"
"-short-bg-color [color], -short-background-color [color]\n"
"        This option specifies the background color of the hour hand.\n"
"\n"
"-long-length [n]\n"
"        This option specifies the length of the minute hand.\n"
"\n"
"-long-fg-thickness [n], -long-foreground-thickness [n]\n"
"        This option specifies the thickness of the minute hand.\n"
"\n"
"-long-bg-thickness [n], -long-background-thickness [n]\n"
"        This option specifies the background thickness of the minute hand.\n"
"\n"
"-short-length [n]\n"
"        This option specifies the length of the hour hand.\n"
"\n"
"-short-fg-thickness [n], -short-foreground-thickness [n]\n"
"        This option specifies the thickness of the hour hand.\n"
"\n"
"-short-bg-thickness [n], -short-background-thickness [n]\n"
"        This option specifies the background thickness of the hour hand.\n"
"\n"
"-center [x] [y]\n"
"        This option specifies the center of the clock.\n"
"\n"
"SEE ALSO\n"
"\n"
"        X(1)\n"
"\n"
"AUTHOR\n"
"\n"
"        Programed by Sakai Hiroaki.\n"
"        E-Mail:sakai@miya.ee.kagu.sut.ac.jp\n"
"\n"
"SPECIAL THANKS\n"
"\n"
"        Tsukuno Sawako for drawing some sample pictures.\n"
"        Tanaka Makoto for giving a sample picture.\n"
"\n"
"COPYRIGHT\n"
"\n"
"        EyeClock Copyright (c) 1998 Sakai Hiroaki.\n"
"        All Rights Reserved.\n"
"\n"
"        BSD Daemon Copyright 1988 by Marshall Kirk McKusick.\n"
"        All Rights Reserved.\n"
"\n"
"        EyeClock is a free software. You may copy or distribute the original\n"
"        EyeClock freely. But, you may not modify or distribute the original\n"
"        EyeClock without permission of the author.\n"
"\n";

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
